Method: Mixlib::ShellOut#run_command
- Defined in:
- lib/mixlib/shellout.rb
#run_command ⇒ Object
Run the command, writing the command’s standard out and standard error to stdout and stderr, and saving its exit status object to status
Returns
returns self; stdout, stderr, status, and exitstatus will be populated with results of the command
Raises
-
Errno::EACCES when you are not privileged to execute the command
-
Errno::ENOENT when the command is not available on the system (or not in the current $PATH)
-
CommandTimeout when the command does not complete within
timeoutseconds (default: 600s)
222 223 224 225 226 227 228 |
# File 'lib/mixlib/shellout.rb', line 222 def run_command if logger = (log_tag.nil? ? "" : "#@log_tag ") << "sh(#@command)" logger.send(log_level, ) end super end |